/**
 * @prettier
 */
type Enumerate<N extends number, Acc extends number[] = []> = Acc["length"] extends N ? Acc[number] : Enumerate<N, [...Acc, Acc["length"]]>;
type IntRange<F extends number, T extends number> = F extends T ? F : Exclude<Enumerate<T>, Enumerate<F>> extends never ? never : Exclude<Enumerate<T>, Enumerate<F>> | T;
export type StatusCode1xx = IntRange<100, 199>;
export type StatusCode2xx = IntRange<200, 299>;
export type StatusCode3xx = IntRange<300, 399>;
export type StatusCode4xx = IntRange<400, 499>;
export type StatusCode5xx = IntRange<500, 599>;
export type StatusCode = StatusCode1xx | StatusCode2xx | StatusCode3xx | StatusCode4xx | StatusCode5xx;
export type Res<Status extends StatusCode, Type> = {
    status: Status;
    json: () => Promise<Type>;
};
export type TypedFetchResponse<R extends Res<any, any>> = Promise<Omit<Response, "json" | "status"> & R>;
export interface AbstractFetchClientConfig {
    basePath: string;
    defaultHeaders: Record<string, string>;
    defaultTimeout?: number;
}
export type QueryParams = {
    [name: string]: string | number | boolean | string[] | undefined | null | QueryParams | QueryParams[];
};
export type HeaderParams = Record<string, string | undefined>;
export declare abstract class AbstractFetchClient {
    protected readonly basePath: string;
    protected readonly defaultHeaders: Record<string, string>;
    protected readonly defaultTimeout?: number;
    protected constructor(config: AbstractFetchClientConfig);
    protected _fetch<R extends Res<any, any>>(url: string, opts: RequestInit, timeout?: number | undefined): TypedFetchResponse<R>;
    protected _query(params: QueryParams): string;
    protected _headers(headers: HeaderParams): Record<string, string>;
}
export {};
